'''
OC Robotique 2025
Template pour librairie Protocole Réseau pour Micro:bit

Auteur·ice : Vincent Namy
Version : 1.0
Date : 29.01.25
'''

#### Libraries ####
from microbit import *
import radio

#### Variables globales ####
seqNum = 0
tryTime = 100
Timeout = 300
ackMsgId = 255

#### Start radio module ####
radio.config(channel=7, address=50)
radio.on()


#### Classe Message ####
class Message:
  def __init__(self, dest:int, exped:int, seqNum:int, msgId:int, payload:List[int], crc:int):
    '''
    Constructeur de l'objet Message à partir des paramètres
            Parameters:
                    dest:int, exped:int, seqNum:int, msgId:int, payload:List[int], crc:int
            Returns:
                    self(Message): objet Message contenant les paramètres
    '''
    self.exped = exped
    self.dest = dest
    self.seqNum = seqNum
    self.msgId = msgId
    self.payload = payload
    self.crc = crc
  def msgStr(self):
    '''
    Crée une string contenant les détails du message
            Parameters:
                    self(Message): objet message
            Returns:
                    msgStr(str): string contenant les détails du message
    '''
    return str(self.exped)+ " -> "+ str(self.dest)+ "n[" + str(self.seqNum)+ "] "+ " : type "+ str(self.msgId)+" : " +str(self.payload)+ " (crc="+ str(self.crc)+")"

#### Toolbox ####
def bytes_to_int(bytesPayload:bytes):
    '''
    Convert bytes object to List[int]
            Parameters:
                    bytesPayload(bytes): payload in bytes format
            Returns:
                    intPayload(List[int]): payload in int format
    '''
    intPayload = []
    for i in bytesPayload:
        intPayload.append(ord(bytes([i])))        
    return intPayload


def int_to_bytes(intPayload:List[int]):    
    '''
    Convert  List[int] to bytes object 
            Parameters:
                    intPayload(List[int]): payload in int format
            Returns:
                    bytesPayload(bytes): payload in bytes format
    '''
    return bytes(intPayload)


#### Fonctions réseaux ####    
def ack_msg(msg : Message):
    '''
    Envoie un ack du message recu.
    1) Création d'une liste de int correspondant au ack dans l'ordre du protocole
    2) Conversion en bytes
    3) Envoi
            Parameters:
                    msg(Message): Objet Message contenant tous les paramètres du message à acker
    '''
    pass # à compléter


def receive_ack(msg: Msg):
    '''
    Attend un ack correspondant au message recu.
    1) Récupère les messages recus
    2) Conversion trame en objet Message
    3) Check si le ack correspond
            Parameters:
                    msg(Message): Objet Message duquel on attend un ack
            Returns:
                    acked(bool): True si message acké, sinon False
    '''
    pass # à compléter
    

def send_msg(msgId:int, payload:List[int], userId:int, dest:int):
    global seqNum
    
    msg = [dest, userId, msgId] + payload
    radio.send_bytes(int_to_bytes(msg))
    

def receive_msg(userId:int):
    
    new_trame = radio.receive_bytes()
    if new_trame:
        trame = bytes_to_int(new_trame)
        msgObj = Message( trame[0], trame[1], None, trame[2], trame[3], None)
        
        if msgObj and msgObj.dest == destId:
            return msgObj

if __name__ == '__main__':
    
    userId = 0
    destId = 2
    
    # Main
    while True:
        # Messages à envoyer
        if button_a.was_pressed():
            send_msg(1,[60],userId, destId)            

                
        # Reception des messages
        m = receive_msg(userId)        
        if m and m.msgId==1 and m.payload == 60:
            display.show(Image.SQUARE)
            sleep(200)
            display.clear()
            
    


